#
# aplicação que mostra propriedades de estimadores da probabilidade e variância de um evento de Bernouilli
#
# autor: Sergio Miranda Freire (Universidade do Estado do Rio de Janeiro)
# 
# License: Creative Commons: CC BY-SA
#

library(shiny)
library(ggplot2)

server <- function(input, output) {

  # probabilidade do evento
  p <- reactive ({
      input$p
  })

  # número de observações
  n_obs <- reactive ({
      input$n_amostras
  })
  
  # plota a proporção acumulada
  output$cum_proportion <- renderPlot({
      # y vai acumulando a proporção à medida que o número de observações aumenta
      y <- cumsum(rbinom(n_obs(), 1, p()))/(1:n_obs())
      g <- ggplot(data.frame(x = 1:n_obs(), y = y), aes(x = x, y = y))
      g <- g + geom_hline(yintercept = 0) + geom_line(size = 2) + geom_hline(yintercept = p(),  col="red") + geom_line(size = 2)
      g <- g + labs(x = "Número de observações", y = "Proporção acumulada")
      g
      })

  # plota a variância da proporção acumulada
  output$cum_proportion_variance <- renderPlot({
      # y vai acumulando a variância à medida que o número de observações aumenta
      y <- cumsum(rbinom(n_obs(),1, p()))/(1:n_obs())
      y <- y * (1-y)
      g <- ggplot(data.frame(x = 1:n_obs(), y = y), aes(x = x, y = y))
      g <- g + geom_hline(yintercept = 0) + geom_line(size = 2) + geom_hline(yintercept = p()*(1-p()), col = "red") + geom_line(size = 2)
      g <- g + labs(x = "Número de observações", y = "Variância Acumulada")
      g
  })

  }

ui <- fluidPage(
  
  titlePanel(expression("Estimadores da probabilidade e variância de um evento de Bernouilli")),
  
  sidebarLayout(
    sidebarPanel(
    sliderInput("n_amostras", 
                label = expression("Número de observações"), 
                min = 10, max = 20000, step = 10,
                value = 5000
    ),
    sliderInput("p", 
                label = expression("Probabilidade de ocorrência do evento de interesse"), 
                min = 0, max = 1, step = .01,
                value = 0.7
    ),
    submitButton("Submeter")
    ),

    mainPanel(
        fluidRow(
            column(6, plotOutput("cum_proportion"))
        ),
        fluidRow(
            column(6, plotOutput("cum_proportion_variance"))
        )
    )
  )
)


shinyApp(ui = ui, server = server)