#
# applicação que mostra a construção da curva ROC
#
# autor: Sergio Miranda Freire (Universidade do Estado do Rio de Janeiro)
#
# Licença: Creative Commons: CC BY-SA
#

library(shiny)
library(RcmdrMisc)

resolution <- 0.1

server <- function(input, output) {

  # ponto de corte para o cálculo da especificidade e sensibilidade
  cutoff <- reactive ({
    input$cutoff
  })

  # média da distribuição dos não doentes
  meanNonDis <- reactive ({
    input$meanNonDis
  })
  
  # desvio padrão da distribuição dos não doentes
  sdNonDis <- reactive ({
    input$sdNonDis
  })
  
  # média da distribuição dos doentes
  meanDis <- reactive ({
    input$meanDis
  })
  
  # desvio padrão da distribuição dos doentes
  sdDis <- reactive ({
    input$sdDis
  })
  
  output$pad <- renderPlot({
    # limites inferior e superior para a construção dos gráficos
    if (meanDis() >= meanNonDis()) {
       inf_limit = meanNonDis() - 4 * sdNonDis()
       sup_limit = meanDis() + 4 * sdDis()
       FPregion = c(cutoff(), Inf)
       FNregion = c(-Inf, cutoff())
       xpos <- meanDis() + (sup_limit - meanDis())/4  # posição da legenda
    } else {
       inf_limit = meanDis() - 4 * sdDis()
       sup_limit = meanNonDis() + 4 * sdNonDis()
       FPregion = c(-Inf, cutoff())
       FNregion = c(cutoff(), Inf)
       xpos <- meanNonDis() + (sup_limit - meanNonDis())/4  # posição da legenda
    }
    adjX <- (meanNonDis() - inf_limit) / (sup_limit - inf_limit)
    x <- seq(inf_limit, sup_limit, resolution)
      # encontra o valor máximo no eixo y para traçar as duas curvas normais em uma mesma escala
    ymax = max(dnorm(meanDis(),meanDis(),sdDis()), dnorm(meanNonDis(),meanNonDis(),sdNonDis()))
      # gráfico da densidade de probabilidade dos não doentes
    par(cex = 1.5)
    plotDistr(x, dnorm(x, meanNonDis(), sdNonDis()), xlab="", ylab="", main="Não Doentes", 
              xlim=c(inf_limit, sup_limit), adj=adjX, yaxt='n', ylim = c(0, ymax), lwd = 2,
              region=list(FPregion), col=rgb(1, 1, 0, .5), yaxt='n', legend = FALSE)
    par(new=TRUE)
    adjX <- (meanDis() - inf_limit) / (sup_limit - inf_limit)
    # gráfico da densidade de probabilidade dos doentes
    if(abs(meanDis() - meanNonDis()) > 50) {  # este teste é feito para evitar a superposição dos rótulos das duas distribuições
      plotDistr(x, dnorm(x, meanDis(),sdDis()), xlab="", ylab="", main="Doentes", 
                xlim=c(inf_limit, sup_limit), adj=adjX, yaxt='n', ylim = c(0, ymax), lwd = 2,
                region=list(FNregion), col=rgb(1, 0, 0, .5), yaxt='n', legend = FALSE)
    } else {
      plotDistr(x, dnorm(x, meanDis(),sdDis()), xlab="", ylab="", xlim=c(inf_limit, sup_limit), 
                main="", adj=adjX, yaxt='n', ylim = c(0, ymax), lwd = 2,
                region=list(FNregion), col=rgb(1, 0, 0, .5), yaxt='n', legend = FALSE)
      mtext("Doentes", adj= adjX, cex = 1.7, font = 2)
    }
    title(xlab=input$test, adj = 0.4)
    limits <- par("usr")
    ypos <- 5* par("usr")[4]/6 # obtém 5/6 do valor máximo do eixo y
    par(cex=1.1)
    legend(xpos, ypos, c("Falsos Positivos","Falsos Negativos"), fill=c("yellow","red"))
  })

  output$roc <- renderPlot({
    # gráfico da curva ROC
    par(mar=c(5,5,1,2), cex = 1.5)
    plot(c(0, 1), c(0, 1), main = "", type = "n", xlab = "", ylab = "")
    if (input$showROC == TRUE) {
      if (meanDis() >= meanNonDis()) {
          curve(pnorm(qnorm(x, meanNonDis(), sdNonDis(), lower.tail=FALSE), meanDis(), sdDis(), lower.tail=FALSE) , xlim=c(0,1),main="", xlab = "", ylab = "", lwd = 2)
      } else {
          curve(pnorm(qnorm(x, meanNonDis(), sdNonDis(), lower.tail=TRUE), meanDis(), sdDis(), lower.tail=TRUE) , xlim=c(0,1),main="", xlab = "", ylab = "", lwd = 2)
      }
    }
    # especificidade e sensibilidade correspondentes ao ponto de corte
    title(xlab = "1 - Especificidade", ylab = "Sensibilidade")
    if (input$showPoint == TRUE) {
      if (meanDis() >= meanNonDis()) {
          x <- pnorm(cutoff(), meanNonDis(), sdNonDis(), lower.tail = FALSE)
          y <- pnorm(cutoff(), meanDis(), sdDis(), lower.tail = FALSE)
      } else {
        x <- pnorm(cutoff(), meanNonDis(), sdNonDis(), lower.tail = TRUE)
        y <- pnorm(cutoff(), meanDis(), sdDis(), lower.tail = TRUE)
      }
      # plotando o ponto (x, y) e mostrando os valores de sensibilidade e especificidade
      points(x, y, col = "blue", pch = 21, bg = "blue")
      text(x + .01, y - .05, paste("S = ", sprintf("%.2f", y)), pos = 4)
      text(x + .01, y - .12, paste("E = ", sprintf("%.2f", 1 - x)), pos = 4)
    }
  })
}


ui <- fluidPage(
  
  titlePanel("Curva ROC"),
  
  sidebarLayout(
    sidebarPanel(
      textInput("diagnosis", label = expression("Diagnóstico"), 
                value = expression("Diabetes")
      ),
      textInput("test", label = expression("Teste"), 
                value = expression("Glicose (mg/dl)")
      ),
      sliderInput("meanNonDis", 
                 label = expression("Média da população não doente"), 
                 min = 60, max = 150,
                 value = 110
    ),
    sliderInput("sdNonDis", 
                 label = expression("Desvio padrão da população não doente"), 
                 min = 10, max = 50,
                 value = 25
    ),
    sliderInput("meanDis", 
                 label = expression("Média da populacao Doente"), 
                 min = 100, max = 200,
                 value = 142
    ),
    sliderInput("sdDis", 
                 label = expression("Desvio padrão da população Doente"), 
                 min = 15, max = 50,
                 value = 30
    ),
    sliderInput("cutoff",
                  "Ponto de corte:",
                  min = 50,
                  max = 190,
                  value = 120),
    checkboxInput(inputId = "showPoint", "Exibir Sensibilidade e Especificidade", value = FALSE),
    checkboxInput(inputId = "showROC", "Exibir Curva ROC", value = FALSE)
    ),
    
    mainPanel(
      fluidRow(
        column(8, plotOutput("pad")),
        column(8, plotOutput("roc"))
      ))
  )
)


shinyApp(ui = ui, server = server)