#
# applicação que mostra a comparação de dois testes de diagnóstico
#
# autor: Sergio Miranda Freire (Universidade do Estado do Rio de Janeiro)
#
# Licença: Creative Commons: CC BY-SA
#

library(shiny)
library(RcmdrMisc)

resolution = 0.1

server = function(input, output) {

  # ponto de corte para o cálculo da especificidade e sensibilidade do teste 1
  cutoff1 = reactive ({
    input$cutoff1
  })

  # média da distribuição dos não doentes no teste 1  
  meanNonDis1 = reactive ({
    input$meanNonDis1
  })
  
  # desvio padrão da distribuição dos não doentes no teste 1
  sdNonDis1 = reactive ({
    input$sdNonDis1
  })
  
  # média da distribuição dos doentes no teste 1  
  meanDis1 = reactive ({
    input$meanDis1
  })
  
  # desvio padrão da distribuição dos doentes no teste 1
  sdDis1 = reactive ({
    input$sdDis1
  })
  
  # ponto de corte para o cálculo da especificidade e sensibilidade do teste 2
  cutoff2 = reactive ({
      input$cutoff2
  })
  
  # média da distribuição dos não doentes no teste 2  
  meanNonDis2 = reactive ({
      input$meanNonDis2
  })
  
  # desvio padrão da distribuição dos não doentes no teste 2
  sdNonDis2 = reactive ({
      input$sdNonDis2
  })
  
  # média da distribuição dos doentes no teste 2  
  meanDis2 = reactive ({
      input$meanDis2
  })
  
  # desvio padrão da distribuição dos doentes no teste 2
  sdDis2 = reactive ({
      input$sdDis2
  })

  inf_limit1 =  reactive ({
    if (meanDis1() >= meanNonDis1()) {
      inf_limit1 = meanNonDis1() - 4 * sdNonDis1()
    } else {
      inf_limit1 = meanDis1() - 4 * sdDis1()
    }
  })

  sup_limit1 =  reactive ({
    if (meanDis1() >= meanNonDis1()) {
      sup_limit1 = meanDis1() + 4 * sdDis1()
    } else {
      sup_limit1 = meanNonDis1() + 4 * sdNonDis1()
    }
  })
  
    # limites inferior e superior para a construção dos gráficos 
  inf_limit2 =  reactive ({
    if (meanDis2() >= meanNonDis2()) {
      inf_limit2 = meanNonDis2() - 4 * sdNonDis2()
    } else {
      inf_limit2 = meanDis2() - 4 * sdDis2()
    }
  })
  
  sup_limit2 =  reactive ({
    if (meanDis2() >= meanNonDis2()) {
      sup_limit2 = meanDis2() + 4 * sdDis2()
    } else {
      sup_limit2 = meanNonDis2() + 4 * sdNonDis2()
    }
  })

  output$test1 = renderPlot({
    if (meanDis1() >= meanNonDis1()) {
      FPregion1 = c(cutoff1(), Inf)
      FNregion1 = c(-Inf, cutoff1())
      xpos <- meanDis1() + 1.5*(sup_limit1() - meanDis1())/5  # posição da legenda
    } else {
      FPregion1 = c(-Inf, cutoff1())
      FNregion1 = c(cutoff1(), Inf)
      xpos <- meanNonDis1() + 1.5*(sup_limit1() - meanNonDis1())/5  # posição da legenda
    }
    x <- seq(inf_limit1(), sup_limit1(), resolution)
    adjX = (meanNonDis1() - inf_limit1()) / (sup_limit1() - inf_limit1())
    # encontra o valor máximo no eixo y para traçar as duas curvas normais em uma mesma escala
    ymax = max(dnorm(meanNonDis1(),meanNonDis1(),sdNonDis1()), dnorm(meanDis1(),meanDis1(),sdDis1()))
    par(cex=1.5)
    plotDistr(x, dnorm(x, meanNonDis1(), sdNonDis1()), xlab="", ylab="", main="Não Doentes", 
              xlim=c(inf_limit1(), sup_limit1()), adj=adjX, yaxt='n', ylim = c(0, ymax), lwd = 2,
              region=list(FPregion1), col=rgb(1, 1, 0, .5), yaxt='n', legend = FALSE)
    par(new=TRUE)
    adjX = (meanDis1() - inf_limit1()) / (sup_limit1() - inf_limit1())
    # gráfico da densidade de probabilidade dos doentes no teste 1
    if(abs(meanDis1() - meanNonDis1()) > 50) {  # este teste é feito para evitar a superposição dos rótulos das duas distribuições
      plotDistr(x, dnorm(x, meanDis1(),sdDis1()), xlab="", ylab="", main="Doentes", 
                xlim=c(inf_limit1(), sup_limit1()), adj=adjX, yaxt='n', ylim = c(0, ymax), lwd = 2,
                region=list(FNregion1), col=rgb(1, 0, 0, .5), yaxt='n', legend = FALSE)
    } else {
      plotDistr(x, dnorm(x, meanDis1(),sdDis1()), xlab="", ylab="", xlim=c(inf_limit1(), sup_limit1()), 
                main="", adj=adjX, yaxt='n', ylim = c(0, ymax), lwd = 2,
                region=list(FNregion1), col=rgb(1, 0, 0, .5), yaxt='n', legend = FALSE)
      mtext("Doentes", adj= adjX, cex = 1.7, font = 2)
    }
    title(xlab=input$test1, adj = 0.4)
    limits = par("usr")
    ypos = 5* par("usr")[4]/6 # obtém 5/6 do valor máximo do eixo y
    par(cex=1.1)
    legend(xpos, ypos, c("Falsos Positivos","Falsos Negativos"), fill=c("yellow","red"))
  })

  output$test2 = renderPlot({
    if (meanDis2() >= meanNonDis2()) {
      FPregion2 = c(cutoff2(), Inf)
      FNregion2 = c(-Inf, cutoff2())
      xpos <- meanDis2() + 1.5*(sup_limit2() - meanDis2())/5  # posição da legenda
    } else {
      FPregion2 = c(-Inf, cutoff2())
      FNregion2 = c(cutoff2(), Inf)
      xpos <- meanNonDis2() + 1.5*(sup_limit2() - meanNonDis2())/5  # posição da legenda
    }
    x <- seq(inf_limit2(), sup_limit2(), resolution)
    adjX = (meanNonDis2() - inf_limit2()) / (sup_limit2() - inf_limit2())
    # encontra o valor máximo no eixo y para traçar as duas curvas normais em uma mesma escala
    ymax = max(dnorm(meanNonDis2(),meanNonDis2(),sdNonDis2()), dnorm(meanDis2(),meanDis2(),sdDis2()))
    par(cex=1.5)
    plotDistr(x, dnorm(x, meanNonDis2(), sdNonDis2()), xlab="", ylab="", main="Não Doentes", 
              xlim=c(inf_limit2(), sup_limit2()), adj=adjX, yaxt='n', ylim = c(0, ymax), lwd = 2,
              region=list(FPregion2), col=rgb(1, 1, 0, .5), yaxt='n', legend = FALSE)
    curve(dnorm(x, meanNonDis2(), sdNonDis2()) , xlim=c(inf_limit2(), sup_limit2()), main="", xlab = "", ylab = "", xaxt = 'n', yaxt = 'n', lwd = 2, col= "blue", add = TRUE)
    # desenho da área correspondente aos falsos positivos do teste 2
    par(new=TRUE)
    adjX = (meanDis2() - inf_limit2()) / (sup_limit2() - inf_limit2())
    # gráfico da densidade de probabilidade dos doentes no teste 2
    if(abs(meanDis2() - meanNonDis2()) > 7) {  # este teste é feito para evitar a superposição dos rótulos das duas distribuições
      plotDistr(x, dnorm(x, meanDis2(),sdDis2()), xlab="", ylab="", main="Doentes", 
                xlim=c(inf_limit2(), sup_limit2()), adj=adjX, yaxt='n', ylim = c(0, ymax), lwd = 2,
                region=list(FNregion2), col=rgb(1, 0, 0, .5), yaxt='n', legend = FALSE)
    } else {
      plotDistr(x, dnorm(x, meanDis2(),sdDis2()), xlab="", ylab="", xlim=c(inf_limit2(), sup_limit2()), 
                main="", adj=adjX, yaxt='n', ylim = c(0, ymax), lwd = 2,
                region=list(FNregion2), col=rgb(1, 0, 0, .5), yaxt='n', legend = FALSE)
      mtext("Doentes", adj= adjX, cex = 1.7, font = 2)
    }
    curve(dnorm(x, meanDis2(), sdDis2()) , xlim=c(inf_limit2(), sup_limit2()), main="", xlab = "", ylab = "", xaxt = 'n', yaxt = 'n', lwd = 2, col= "blue", add = TRUE)
    title(xlab=input$test2, adj = 0.4)
    limits = par("usr")
    ypos = 5* par("usr")[4]/6 # obtém 5/6 do valor máximo do eixo y
    par(cex=1.1)
    legend(xpos, ypos, c("Falsos Positivos","Falsos Negativos"), fill=c("yellow","red"))
  })
  
  output$roc = renderPlot({
    par(mar=c(5,5,1,2))
    # curva ROC, e especificidade e sensibilidade correspondentes ao ponto de corte do teste 1
    par(cex=1.5)
    if (meanDis1() >= meanNonDis1()) {
        curve(pnorm(qnorm(x, meanNonDis1(), sdNonDis1(), lower.tail=FALSE), meanDis1(), sdDis1(), lower.tail=FALSE) ,xlim=c(0,1), main="", xlab = "", ylab = "", col = "black", lwd = 2)
        x = pnorm(cutoff1(),meanNonDis1(), sdNonDis1(), lower.tail=FALSE)
        y = pnorm(cutoff1(),meanDis1(), sdDis1(), lower.tail=FALSE)
    } else {
        curve(pnorm(qnorm(x, meanNonDis1(), sdNonDis1(), lower.tail=TRUE), meanDis1(), sdDis1(), lower.tail=TRUE) , xlim=c(0,1), main="", xlab = "", ylab = "", col = "black", lwd = 2)
        x <- pnorm(cutoff1(), meanNonDis1(), sdNonDis1(), lower.tail = TRUE)
        y <- pnorm(cutoff1(), meanDis1(), sdDis1(), lower.tail = TRUE)
    }
    # plotando o ponto (x, y) e mostrando os valores de sensibilidade e especificidade do teste 1
    points(x, y, col = "black", pch=21, cex = 2.0, bg = "black")
    text(x +.01, y +.07, paste("S = ", sprintf("%.2f", y)), pos = 4)
    text(x +.01, y +.13, paste("E = ", sprintf("%.2f", 1-x)), pos = 4)

    # curva ROC, especificidade e sensibilidade correspondentes ao ponto de corte do teste 2
    if (meanDis2() >= meanNonDis2()) {
        curve(pnorm(qnorm(x, meanNonDis2(), sdNonDis2(), lower.tail=FALSE), meanDis2(), sdDis2(), lower.tail=FALSE) ,xlim=c(0,1),main="", xlab = "", ylab = "", col = "blue", lwd = 2, add = T)
        x <- pnorm(cutoff2(), meanNonDis2(), sdNonDis2(), lower.tail = FALSE)
        y <- pnorm(cutoff2(), meanDis2(), sdDis2(), lower.tail = FALSE)
    } else {
        curve(pnorm(qnorm(x, meanNonDis2(), sdNonDis2(), lower.tail=TRUE), meanDis2(), sdDis2(), lower.tail=TRUE) ,xlim=c(0,1),main="", xlab = "", ylab = "", col = "blue", lwd = 2, add = T)
        x <- pnorm(cutoff2(), meanNonDis2(), sdNonDis2(), lower.tail = TRUE)
        y <- pnorm(cutoff2(), meanDis2(), sdDis2(), lower.tail = TRUE)
    }
    # plotando o ponto (x, y) e mostrando os valores de sensibilidade e especificidade do teste 2
    points(x, y, col = "blue", pch=21, cex = 2.0, bg = "black")
    text(x - .01, y -.13, paste("S = ", sprintf("%.2f", y)), pos = 4)
    text(x - .01, y -.07, paste("E = ", sprintf("%.2f", 1-x)), pos = 4)
    title(xlab = "1 - Especificidade", ylab = "Sensibilidade")
    legend(.5, .5, c(input$test1, input$test2), fill=c("black","blue"))
  })
}


ui = fluidPage(
  
  titlePanel("Curvas ROC para dois testes"),
  
  sidebarLayout(
    sidebarPanel(
      textInput("diagnosis", label = expression("Diagnóstico"), 
                value = expression("Diabetes")
      ),
      textInput("test1", label = expression("Teste 1"), 
                value = expression("Glicose (mg/dl)")
      ),
      sliderInput(
            "meanNonDis1",
            label = expression("Média da população Não Doente Teste 1"),
            min = 60, max = 150,
            value = 110
        ),
        sliderInput(
            "sdNonDis1",
            label = expression("Desvio padrão da população não doente Teste 1"),
            min = 10, max = 50,
            value = 25
        ),
        sliderInput(
            "meanDis1",
            label = expression("Média da população doente Teste 1"),
            min = 100, max = 200,
            value = 142
        ),
        sliderInput(
            "sdDis1",
            label = expression("Desvio padrão da população doente Teste 1"),
            min = 15, max = 50,
            value = 30
          ),
        sliderInput(
            "cutoff1",
            "Ponto de corte Teste 1",
            min = 30,
            max = 250,
            value = 120
        ),
        textInput("test2", label = expression("Teste 2"), 
                  value = expression("Índice de massa corporal (kg/m2)")
        ),
        sliderInput(
            "meanNonDis2",
            label = expression("Média da população não doente teste 2"),
            min = 15,
            max = 60,
            value = 31
        ),
        sliderInput(
            "sdNonDis2",
            label = expression("Desvio padrão da população não doente Teste 2"),
            min = 2,
            max = 15,
            value = 6.5
        ),
        sliderInput(
            "meanDis2",
            label = expression("Média da população doente Teste 2"),
            min = 15,
            max = 60,
            value = 35
        ),
        sliderInput(
            "sdDis2",
            label = expression("Desvio padrão da população doente Teste 2"),
            min = 2,
            max = 15,
            value = 6.5
        ),
        sliderInput(
            "cutoff2",
            "Ponto de corte Teste 2",
            min = 0,
            max = 100,
            value = 33
        )
    ),
    
    # Show plots
    mainPanel(fluidRow(
        column(8, plotOutput("test1")),
        column(8, plotOutput("test2")),
        column(8, plotOutput("roc"))
    ))
  )
)


shinyApp(ui = ui, server = server)