#
# aplicação que traça a curva de probabilidade pós-teste de doença em função da prevalência,
# tomando a razão de verossimilhança como parâmetro. 
#
# autor: Sergio Miranda Freire (Universidade do Estado do Rio de Janeiro)
#
# Licença: Creative Commons: CC BY-SA#
#

library(shiny)

# Função para o cálculo da probabilidade de doença pós-teste em função da 
# razão de verossimilhança e prevalência da doença
post_test_probability <- function(prev, likelihood) {
    pre_odds = prev / (1 - prev)
    post_odds = pre_odds * likelihood
    post_odds / (1 + post_odds)
}

# Define server logic required to draw the curve
server <- function(input, output) {

    likelihood <- reactive ({
        input$likelihood
    })

    pre_test_prob <- reactive ({
        input$pre_test_prob
    })
    
    output$post_pre_test_prob <- renderPlot({
        curve(post_test_probability(x, likelihood()), xlim = c(0, .999), main = "", xlab = "", ylab = "")
        par(cex = 1.1)
        title(xlab = "Probabilidade Pré-teste", ylab = "Probabilidade Pós-teste")
        x =  pre_test_prob()
        y = post_test_probability(x, likelihood())
        points(x, y, col = "blue", pch = 21, cex = 1.0, bg = "blue")
        if (y > .9) {
            text(x - .15, y-.1, paste("P(D|T) = ", sprintf("%.2f", y)), pos = 4)
        } else if (y < .1) {
            text(x, y + .05, paste("P(D|T) = ", sprintf("%.2f", y)), pos = 4)
        } else {
            text(x + .01, y - .05, paste("P(D|T) = ", sprintf("%.2f", y)), pos = 4)
        }
    })
    
}

ui <- fluidPage(
    titlePanel("Probabilidade Pós-Teste x Probabilidade Pré-teste"),
    
    sidebarLayout(
        sidebarPanel(
            numericInput("likelihood", 
                label = expression("Razão de verossimilhança"),
                min = 0.001, max = 100, step = .1, value = 1
            ),
            sliderInput("pre_test_prob",
                label = expression("Probabilidade pré-teste"),
                min = 0, max = .999, value = 0.7
            )
        ),

        mainPanel(plotOutput("post_pre_test_prob"))
    )
)

shinyApp(ui = ui, server = server)