#  
#   Teste de randomização e Intervalo de Confiança
#
#  Ilustra o conceito de teste de hipótese, valor de p e intervalo de confiança
#  baseado em testes de randomização  
#
# autor: Sergio Miranda Freire (Universidade do Estado do Rio de Janeiro)
# 
# License: Creative Commons: CC BY-SA
#

library(shiny)
library(stats)
library(shinyjs)

server <- function(input, output) {

  parametros <- reactiveValues(x = NULL, y = NULL, gx_nome = "", gy_nome = "", p = 0, variavel = "", 
                       diff = 0, total_sample = NULL, n1 = 0, sample_size = 0, alfa_2 = 0.025)

observeEvent(input$descritiva, {
  output$graficoDescritivo <- renderPlot({
    parametros$alfa_2 = (100 - as.numeric(substring(input$confianca, 0, 2))) / 200
    g1_valores = as.numeric(strsplit(input$valoresGrupo1, ",")[[1]])
    g2_valores = as.numeric(strsplit(input$valoresGrupo2, ",")[[1]])
    if ((mean(g1_valores) - mean(g2_valores)) < 0) {
      parametros$x = g2_valores
      parametros$y = g1_valores
      parametros$gx_nome = input$grupo2
      parametros$gy_nome = input$grupo1
    } else {
      parametros$x = g1_valores
      parametros$y = g2_valores
      parametros$gx_nome = input$grupo1
      parametros$gy_nome = input$grupo2
    }
    parametros$n1 = length(parametros$x)
    parametros$total_sample = c(parametros$x, parametros$y)
    parametros$diff = mean(parametros$x) - mean(parametros$y)
    parametros$sample_size = length(parametros$total_sample)
    parametros$variavel = input$variavel
    stripchart(list(x = parametros$x, y = parametros$y), vertical=TRUE, group.names =  c(parametros$gx_nome, parametros$gy_nome), method="stack", ylab=parametros$variavel)
    enable("testeHipotese")
    enable("valorP")
    enable("ic_limInferior")
    enable("ic_limSuperior")
      })
})       

observeEvent(input$testeHipotese, {
    output$histTesteHipotese <- renderPlot({
        total_sample = parametros$total_sample
        n1 = parametros$n1
        sample_size = parametros$sample_size
        alfa_2 = parametros$alfa_2
        diff = parametros$diff
        n = 100000
        diferenca = vector("numeric", n)
        diferenca[n] = diff
        m = n-1
        for (i in 1:m) {
            sample = sample(total_sample, sample_size)
            x_sample = sample[1:n1]
            first_y = n1 + 1
            y_sample = sample[(n1+1):sample_size]
            diferenca[i] = mean(x_sample) - mean(y_sample)
        }
        h = hist(diferenca)
        amp = h$breaks[2]-h$breaks[1]
        area = 0
        for (j in 1:length(h$breaks)) {
            area_classe = h$counts[j]/n
            area_ant = area
            area = area_classe + area
            if (area > alfa_2) {
                crit_inf = h$breaks[j] + amp * (alfa_2 - area_ant) / area_classe
                ind_crit_inf = j
                break;  
            } 
        }
        area = 0
        for (j in length(h$breaks):1) {
            area_classe = h$counts[j-1] / n
            area_ant = area
            area = area_classe + area
            if (area > alfa_2) {
                crit_sup = h$breaks[j] - amp * (alfa_2 - area_ant) / area_classe
                ind_crit_sup = j
                break
                
            }
        }
        cuts <- cut(h$breaks, c(-Inf, h$breaks[ind_crit_inf-1], h$breaks[ind_crit_sup-1], Inf))
        plot(h, freq= FALSE, col=c("red", "white", "red")[cuts], ylab = "Densidade", xlab="diferença entre grupos", main = "Histograma Hipótese Nula")
        polygon(c(h$breaks[ind_crit_inf], h$breaks[ind_crit_inf], crit_inf, crit_inf), c(0, h$density[ind_crit_inf], h$density[ind_crit_inf], 0), col="red")
        polygon(c(crit_sup, crit_sup, h$breaks[ind_crit_sup], h$breaks[ind_crit_sup]), c(0, h$density[ind_crit_sup-1], h$density[ind_crit_sup-1], 0), col="red")
        abline(v = c(crit_inf, 0, crit_sup), col = c("red", "black", "red"), lwd = 3)
        legend("topright", legend = c(paste("Região Crítica = ", sprintf("%.2f", alfa_2 * 200), "%"), paste("lim inf = ", sprintf("%.2f", crit_inf)),
                                      paste("lim sup = ", sprintf("%.2f", crit_sup))), col=c(rep("red", 3)), pch=c(15, NA, NA), pt.cex=2, lty = c(0, 1, 1))    
    })
})       

observeEvent(input$valorP, {
  output$histValorP <- renderPlot({
    total_sample = parametros$total_sample
    n1 = parametros$n1
    sample_size = parametros$sample_size
    alfa_2 = parametros$alfa_2
    diff = parametros$diff
    n = 100000
    diferenca = vector("numeric", n)
    diferenca[n] = diff
    m = n-1
    for (i in 1:m) {
      sample = sample(total_sample, sample_size)
      x_sample = sample[1:n1]
      first_y = n1 + 1
      y_sample = sample[(n1+1):sample_size]
      diferenca[i] = mean(x_sample) - mean(y_sample)
    }
    h = hist(diferenca)
    amp = h$breaks[2]-h$breaks[1]
    area = 0
    for (j in length(h$breaks):1) {
      area_classe = h$counts[j-1] / n
      area_ant = area
      area = area_classe + area
      if (area > alfa_2) {
        crit_sup = h$breaks[j] - amp * (alfa_2 - area_ant) / area_classe
        break
      }
    }
    indice_diff = min(which(h$breaks > diff))
    cuts <- cut(h$breaks, c(-Inf, h$breaks[indice_diff-1], Inf))
    plot(h, freq = FALSE, col=c("white", "blue")[cuts], ylab = "Densidade", xlab="diferença entre grupos", main = "Histograma Hipótese Nula")
    polygon(c(diff, diff, h$breaks[indice_diff], h$breaks[indice_diff]), c(0, h$density[indice_diff-1], h$density[indice_diff-1], 0), col="blue")
    abline(v = c(diff, 0, crit_sup), col = c("blue", "black", "red"), lwd = 3)
    p = sum(diferenca >= diff)/n
    parametros$p = p
    if (p < 0.01 ) {
      legend("topleft", legend = c("limite região crítica", paste("dif. obs. = ", sprintf("%.2f", diff)), "p/2 < 0,01"), col=c("red", "blue", "blue"), pch=c(NA, NA, 15),
             pt.cex=2, lty = c(1, 1, 0))
    } else {
      legend("topleft", legend = c("limite região crítica", paste("dif. obs. = ", sprintf("%.2f", diff)), paste("p/2 = ", sprintf("%.3f", p))), col=c("red", "blue", "blue"), pch=c(NA, NA, 15),
             pt.cex=2, lty = c(1, 1, 0))
    }
  })
})       

observeEvent(input$ic_limInferior, {
  output$histIcInferior <- renderPlot({
    sample_size = parametros$sample_size
    x = parametros$x
    y = parametros$y
    n1 = parametros$n1
    diff = parametros$diff
    p = parametros$p
    alfa_2 = parametros$alfa_2
    n = 50000
    m = n - 1
    diferenca = vector("numeric", n)
    # limite inferior
    if (p < alfa_2) {
      lim_sup = diff
      current = diff / 2
      lim_inf = 0
    } else {
      lim_sup = 0
      lim_inf = 0 - 2 * diff
      current = -diff
    }
    p_lower = p
    while (abs(p_lower - alfa_2) > 0.003) {
      x_test = x - current
      total_sample = c(x_test, y)
      diferenca[n] = mean(x_test) - mean(y)
      for (i in 1:m) {
        sample = sample(total_sample, sample_size)
        x_sample = sample[1:n1]
        y_sample = sample[(n1+1):sample_size]
        diferenca[i] = mean(x_sample) - mean(y_sample)
      }
      p_lower = sum((diferenca + current) >= diff) / n
      if (p_lower < alfa_2) {
        lim_inf = current
        current = (lim_sup + current) / 2
      }  else {
        lim_sup = current
        current = (lim_inf + current) / 2
      }
    }
    lower_lim = current
    h = hist(diferenca + current)
    indice_diff = min(which(h$breaks > diff))
    cuts <- cut(h$breaks, c(-Inf, h$breaks[indice_diff-1], Inf))
    plot(h, freq = FALSE, col = c("white", "red")[cuts], ylab = "Densidade", xlab="diferença entre grupos", main = "Limite Inferior IC")
    polygon(c(diff, diff, h$breaks[indice_diff], h$breaks[indice_diff]),
            c(0, h$density[indice_diff - 1], h$density[indice_diff - 1], 0),
            col = "red")
    abline(v = c(diff, lower_lim), col = c("red", "darkgreen"), lwd = 3)
    legend("topleft", legend = c(paste("Área = ", sprintf("%.2f", parametros$alfa_2 * 100), "%"), paste("lim. inf. = ", sprintf("%.2f", lower_lim)),
                                 paste("dif. obs. = ", sprintf("%.2f", diff))),  col=c("red", "darkgreen", "red"), pch=c(15, NA, NA), pt.cex=2, lty = c(0, 1, 1))   
  })
})       

observeEvent(input$ic_limSuperior, {
  output$histIcSuperior <- renderPlot({
    sample_size = parametros$sample_size
    x = parametros$x
    y = parametros$y
    n1 = parametros$n1
    diff = parametros$diff
    p = parametros$p
    alfa_2 = parametros$alfa_2
    n = 50000
    m = n - 1
    diferenca = vector("numeric", n)
    lim_sup = diff * 3
    lim_inf = diff
    current = diff * 2
    p_upper = p
    while (abs(p_upper - alfa_2) > 0.003) {
      x_test = x - current
      total_sample = c(x_test, y)
      diferenca[n] = mean(x_test) - mean(y)
      for (i in 1:m) {
        sample = sample(total_sample, sample_size)
        x_sample = sample[1:n1]
        y_sample = sample[(n1+1):sample_size]
        diferenca[i] = mean(x_sample) - mean(y_sample)
      }
      p_upper = sum((diferenca + current) <= diff) / n
      if (p_upper < alfa_2) {
        lim_sup = current
        current = (lim_inf + current) / 2
      }  else {
        lim_inf = current
        current = (lim_sup + current) / 2
      }
    }
    upper_lim = current
    h = hist(diferenca + current)
    indice_diff = max(which(h$breaks < diff))
    cuts <- cut(h$breaks, c(-Inf, h$breaks[indice_diff - 1], Inf))
    plot(h, freq = FALSE, col = c("red", "white")[cuts], ylab = "Densidade", xlab="diferença entre grupos", main = "Limite Superior IC")
    polygon(c(h$breaks[indice_diff], h$breaks[indice_diff], diff, diff),
            c(0, h$density[indice_diff], h$density[indice_diff], 0),
            col = "red")
    abline(v = c(diff, upper_lim), col = c("red", "darkgreen"), lwd = 3)
    legend("topright", legend = c(paste("Área = ", sprintf("%.2f", parametros$alfa_2 * 100), "%"), paste("lim. sup. = ", sprintf("%.2f", upper_lim)),
                                 paste("dif. obs. = ", sprintf("%.2f", diff))), col=c("red", "darkgreen", "red"), pch=c(15, NA, NA), pt.cex=2, lty = c(0, 1, 1))   
  })
})       

observeEvent(input$reset, {
  output$graficoDescritivo <- NULL
  output$histTesteHipotese <- NULL
  output$histIcInferior <- NULL
  output$histIcSuperior <- NULL
  output$histValorP <- NULL
})       

}

ui <- fluidPage(
  
  titlePanel(expression("Teste de Hipótese e Intervalo de Confiança")),
  
  sidebarLayout(

    sidebarPanel(
      useShinyjs(),
      textInput("variavel", label = expression(Variável), 
                value = "Ácido Fólico"
      ),
      textInput("grupo1", label = "Grupo 1", 
                value = "N2O+O2,24h"
      ),
      textInput("grupo2", label = "Grupo 2", 
                value = "N2O+O2,op"
      ),
      textInput("valoresGrupo1", label = "Valores no Grupo 1", 
                value = "243, 251, 275, 291, 347, 354, 380, 392"
      ),
      textInput("valoresGrupo2", label = "Valores no Grupo 2", 
                value = "206, 210, 226, 249, 255, 273, 285, 295, 309"
      ),
      selectInput("confianca", 
                  label = expression("Nível de confiança:"),
                  choices = c("90%", "95%", "99%"),
                  selected = "95%"),
      actionButton(inputId = "descritiva", label = "Mostrar um diagrama Stripchart"),
      br(),
      br(),
      disabled(
        actionButton(inputId = "testeHipotese", label = "Distribuição sob a hipótese nula"),
        br(),
        br(),
        actionButton(inputId = "valorP", label = "Testar hipótese nula"),
        br(),
        br(),
        actionButton(inputId = "ic_limInferior", label = "Limite inferior do Intervalo de Confiança"),
        br(),
        br(),
        actionButton(inputId = "ic_limSuperior", label = "Limite superior do Intervalo de Confiança")
      ),
      br(),
      br(),
      actionButton(inputId = "reset", label = "Limpar")    
  ),
    
    mainPanel(
      fluidRow(
        column(5, plotOutput("graficoDescritivo")),
        column(5, plotOutput("histValorP"))
      ),
      fluidRow(
        column(5, plotOutput("histTesteHipotese")),
        column(5, plotOutput("histIcInferior"))
      ),
      fluidRow(
          column(5, plotOutput("nothing")),
          column(5, plotOutput("histIcSuperior"))
      )
      
    )
  )
)

shinyApp(ui = ui, server = server)